@@ -35,6 +35,7 @@ func NewTextHandler(w io.Writer) *TextHandler {
 
 type TextHandler struct {
 	w         io.Writer
+	sbom      *govulncheck.SBOM
 	osvs      []*osv.Entry
 	findings  []*findingSummary
 	scanLevel govulncheck.ScanLevel
@@ -63,6 +64,9 @@ const (
 )
 
 func (h *TextHandler) Flush() error {
+	if h.showVerbose {
+		h.printSBOM()
+	}
 	if len(h.findings) == 0 {
 		h.print(noVulnsMessage + "\n")
 	} else {
@@ -115,6 +119,53 @@ func (h *TextHandler) Config(config *govulncheck.Config) error {
 	return h.err
 }
 
+func (h *TextHandler) SBOM(sbom *govulncheck.SBOM) error {
+	h.sbom = sbom
+	return nil
+}
+
+func (h *TextHandler) printSBOM() error {
+	if h.sbom == nil {
+		h.print("No packages matched the provided pattern.\n")
+		return nil
+	}
+
+	printed := false
+
+	for i, root := range h.sbom.Roots {
+		if i == 0 {
+			if len(h.sbom.Roots) > 1 {
+				h.print("The package pattern matched the following ", len(h.sbom.Roots), " root packages:\n")
+			} else {
+				h.print("The package pattern matched the following root package:\n")
+			}
+		}
+
+		h.print("  ", root, "\n")
+		printed = true
+	}
+	for i, mod := range h.sbom.Modules {
+		if i == 0 && mod.Path != "stdlib" {
+			h.print("Govulncheck scanned the following ", len(h.sbom.Modules)-1, " modules and the ", h.sbom.GoVersion, " standard library:\n")
+		}
+
+		if mod.Path == "stdlib" {
+			continue
+		}
+
+		h.print("  ", mod.Path)
+		if mod.Version != "" {
+			h.print("@", mod.Version)
+		}
+		h.print("\n")
+		printed = true
+	}
+	if printed {
+		h.print("\n")
+	}
+	return nil
+}
+
 // Progress writes progress updates during govulncheck execution.
 func (h *TextHandler) Progress(progress *govulncheck.Progress) error {
 	if h.showVerbose {
@@ -297,39 +348,70 @@ func (h *TextHandler) traces(traces []*findingSummary) {
 		return symbol(traces[i].Trace[0], true) < symbol(traces[j].Trace[0], true)
 	})
 
-	first := true
-	count := 1
-	for _, entry := range traces {
-		if entry.Compact == "" {
-			continue // skip package and module level traces
+	// compacts are finding summaries with compact traces
+	// suitable for non-verbose textual output. Currently,
+	// only traces produced by symbol analysis.
+	var compacts []*findingSummary
+	for _, t := range traces {
+		if t.Compact != "" {
+			compacts = append(compacts, t)
 		}
-		if first {
-			if h.scanMode == govulncheck.ScanModeBinary {
+	}
+
+	// binLimit is a limit on the number of binary traces
+	// to show. Traces for binaries are less interesting
+	// as users cannot act on them and they can hence
+	// spam users.
+	const binLimit = 5
+	binary := h.scanMode == govulncheck.ScanModeBinary
+	for i, entry := range compacts {
+		if i == 0 {
+			if binary {
 				h.style(keyStyle, "    Vulnerable symbols found:\n")
 			} else {
 				h.style(keyStyle, "    Example traces found:\n")
 			}
 		}
-		first = false
 
-		h.print("      #", count, ": ")
-		count++
-		if !h.showTraces {
+		// skip showing all symbols in binary mode unless '-show traces' is on.
+		if binary && (i+1) > binLimit && !h.showTraces {
+			h.print("      Use '-show traces' to see the other ", len(compacts)-binLimit, " found symbols\n")
+			break
+		}
+
+		h.print("      #", i+1, ": ")
+
+		if !h.showTraces { // show summarized traces
 			h.print(entry.Compact, "\n")
+			continue
+		}
+
+		if binary {
+			// There are no call stacks in binary mode
+			// so just show the full symbol name.
+			h.print(symbol(entry.Trace[0], false), "\n")
 		} else {
 			h.print("for function ", symbol(entry.Trace[0], false), "\n")
 			for i := len(entry.Trace) - 1; i >= 0; i-- {
 				t := entry.Trace[i]
 				h.print("        ")
+				h.print(symbolName(t))
 				if t.Position != nil {
-					h.print(posToString(t.Position), ": ")
+					h.print(" @ ", symbolPath(t))
 				}
-				h.print(symbol(t, false), "\n")
+				h.print("\n")
 			}
 		}
 	}
 }
 
+// symbolPath returns a user-friendly path to a symbol.
+func symbolPath(t *govulncheck.Frame) string {
+	// Add module path prefix to symbol paths to be more
+	// explicit to which module the symbols belong to.
+	return t.Module + "/" + posToString(t.Position)
+}
+
 func (h *TextHandler) summary(c summaryCounters) {
 	// print short summary of findings identified at the desired level of scan precision
 	var vulnCount int
